/* table.c  generate forward kinematic table and environment
 * 12 Mar 96
 * simplified  4 Mar 97
 * P. Chongstitvatana
 */

#include "gp.h"

extern point elbow, wrist, tip, target;
extern int	 zshoulder, zelbow, zwrist;
extern int	 NearGoal;

/*  graphic routines to create an environment image for simulation */

PRIVATE fillbox(x,y,w,h,v) /* create a filled box with v */
int x,y,w,h,v;
{
  int i,j;
  for(i=x; i<x+w; i++)
    for(j=y; j<y+h; j++) im[i][j] = v;
}

PRIVATE circle(x,y,r,v)
int x,y,r,v;
{
  int r2,xy2,x0,y0,i,t;
  
  x0 = x; y0 = y; 
  r2 = r*r; xy2 = r2;
  x = r; y = -1;      /* to start the first point */
  while ( x > y ) {
    y++;
    xy2 += 2*y + 1;
    if ( xy2 > r2 ) { 
      x--;
      xy2 -= 2*x + 1;
    }
    for( i=0; i<2; i++ ) {
      im[x0+x][y0+y] = v;
      im[x0-x][y0+y] = v;
      im[x0+x][y0-y] = v;
      im[x0-x][y0-y] = v;
      t = x; x = y; y = t;
    }
  }
}
  
PRIVATE fill(type,x,y,v) /* type 0 middle, 1 fillup, -1 filldown */
int type,x,y,v;
{
  int i,xleft,xright;

  for( i=x; i>0 && im[i][y] == BACKGR ; i-- )  ;
  xleft = i+1;
  for( i=xleft; i<250 && im[i][y] == BACKGR; i++ ) im[i][y] = v;
  xright = i;
  switch( type ) {
  case 0:
    for( i=xleft; i<xright && im[i][y+1] != BACKGR; i++ ) ;
    if( i == xright ) return;
    fill(1,i,y+1,v);
    for( i=xleft; i<xright && im[i][y-1] != BACKGR; i++ ) ;
    if( i == xright ) return;
    fill(-1,i,y-1,v);
    break;
  case 1:
  case -1:
    for( i=xleft; i<xright && im[i][y+type] != BACKGR; i++ ) ;
    if( i == xright ) return;
    fill(type,i,y+type,v);
  }
}


/* **** create  environment ************  */

PRIVATE clearimage(v)  /* fill image with v */
int v;
{
  int i,j;
  for(i=0; i<IMW; i++)
    for(j=0; j<IMH; j++) im[i][j] = v;
}

PRIVATE maketarget(x,y,r)
int x,y,r;
{
  target.x = x; target.y = y;
  NearGoal = r*r;
}

env0()
{
  clearimage(BACKGR);
  fillbox(180,60,30,20,OBSTAC);
  maketarget(169,22,10);
}

env1()
{
  clearimage(BACKGR);
  fillbox(115,53,60,20,OBSTAC);
  maketarget(116,33,10);
}

env2()
{
  clearimage(BACKGR);
  fillbox(37,57,45,22,OBSTAC);
  fillbox(137,52,60,20,OBSTAC);
  maketarget(75,35,10);
}

env3()
{
  clearimage(BACKGR);
  fillbox(35,60,50,20,OBSTAC);
  fillbox(140,60,60,20,OBSTAC);
  maketarget(75,30,10);
}

env4()
{
  clearimage(BACKGR);
  circle(140,60,15,OBSTAC);
  fill(0,140,60,OBSTAC);
  maketarget(130,25,10);
}
  
env7() /* free space */
{
  clearimage(BACKGR);
  maketarget(100,50,10);
}

env8()
{
  fillbox(35,60,50,20,OBSTAC);
  fillbox(140,60,60,20,OBSTAC);
  maketarget(130,20,10);
}

env9()
{
  fillbox(20,50,50,20,OBSTAC);
  fillbox(115,50,15,15,OBSTAC);
  fillbox(170,50,60,20,OBSTAC);
  maketarget(120,30,10);
}


/* ********  generate forward kinematics table ******** */

#define pi      3.14159265359
#define pi180   0.0174532925199   /* pi/180 */
#define rad(z)  (double)((z) * pi180 )

int outb(p,x,y,w,h) /* out of bound ? */
point p;
int x,y,w,h;
{
  return ( p.x < x || p.y < y || p.x >= x+w || p.y >= y+h );
}

/* given arm angle, update arm position : 
	 (int) zshoulder,zelbow,zwrist -> (point) elbow,wrist,tip */
armpos()
{
  elbow.x = (short)(L1*cos( rad(zshoulder) ) + SX);
  elbow.y = (short)(-L1*sin( rad(zshoulder) ) + SY);
  wrist.x = (short)(L2*cos( rad(zshoulder+zelbow) )) + elbow.x;
  wrist.y = (short)(-L2*sin( rad(zshoulder+zelbow) )) + elbow.y;
  tip.x = (short)(L3*cos( rad(zshoulder+zelbow+zwrist) )) + wrist.x;
  tip.y = (short)(-L3*sin( rad(zshoulder+zelbow+zwrist) )) + wrist.y;
}









